import abc

from m3 import OperationResult
from objectpack.actions import BaseAction
from objectpack.actions import BasePack
from objectpack.actions import BaseWindowAction


def _download_result(url):
    """
    Результат - скачивание файла.
    """
    return OperationResult(
        success=True,
        code=f"""
            (function(){{
                var hiddenIFrameID = 'hiddenDownloader',
                    iframe = document.getElementById(hiddenIFrameID);
                if (iframe === null) {{
                    iframe = document.createElement('iframe');
                    iframe.id = hiddenIFrameID;
                    iframe.style.display = 'none';
                    document.body.appendChild(iframe);
                }}
                iframe.src = "{url}";
            }})()
        """
    )


class ReportPack(BasePack):
    """
    Пак для отчётов

    :param input_action: Экшн ввода параметров отчёта
    :param report_action: Экшн формирования отчёта
    """

    menu_dir = 'Отчёты'

    def __init__(self, input_action, report_action):
        super(ReportPack, self).__init__()

        self.report_input_action = input_action
        self.report_action = report_action

        self.actions.extend((
            self.report_input_action,
            self.report_action
        ))

    def extend_menu(self, menu):
        """
        Добавляет отчет в меню Пуск.
        """
        menu_item = menu.Item(self.verbose_name,
                              self.report_input_action,
                              icon='report_item')

        return menu.SubMenu(self.menu_dir, menu_item, icon='menu-dicts-16')


class BaseReportInputAction(BaseWindowAction):
    """
    Базовый экшн ввода параметров отчёта
    """

    verbose_name = 'Параметры отчета'
    report_window = None

    def __init__(self):
        super(BaseReportInputAction, self).__init__()
        self.win = None
        self.win_params = {}

    def create_window(self):
        if self.report_window is None:
            raise NotImplementedError

        self.win = self.report_window()

    def set_window_params(self):
        super(BaseReportInputAction, self).set_window_params()
        self.win_params['title'] = self.verbose_name
        self.win_params['form_url'] = (
            self.parent.report_action.get_absolute_url())


class BaseReportAction(abc.ABC, BaseAction):
    """
    Базовый экшн формирования отчёта.
    """

    verbose_name = 'Формирование отчета'
    reporter_cls = None

    @abc.abstractmethod
    def _get_report_params(self, request, context):
        """
        Получение параметров отчёта
        """
        raise NotImplementedError

    def _get_reporter(self):
        """
        Получение составителя отчёта.
        """
        if self.reporter_cls is None:
            raise NotImplementedError

        return self.reporter_cls()

    def run(self, request, context):
        report_params = self._get_report_params(request, context)
        reporter = self._get_reporter()
        _, url = reporter.get_report(report_params)
        return _download_result(url)
