import os
import unittest
from tempfile import NamedTemporaryFile
from tempfile import gettempdir

from openpyxl import load_workbook
from report_writer.core.content.samples.getters import KeyContent
from report_writer.core.report_objects.samples.simple import Cell
from report_writer.impl.xlsx.report.writers.simple import CellWriter
from report_writer.utils.read_utils import read_xlsx

from m3_report_writer.builders import XlsxReportBuilder
from m3_report_writer.reporter import BaseReporter


# На Windows файл нельзя будет удалить
_DELETE_TMP_FILE = os.name != 'nt'
_TMP_DIR = gettempdir()


class Builder(XlsxReportBuilder):
    template_path = os.path.join(
        os.path.dirname(__file__), 'template.xlsx'
    )

    def _init_report_objects(self):
        cell = Cell(CellWriter(self._report))
        cell_content = KeyContent('foo')

        self._add_report_object('cell', cell, cell_content)

    def _get_template_context(self, data):
        return {'value': data['bar']}

    def _build(self):
        cursor = self._report.get_cursor('main')
        self._write_report_object('cell', cursor)


class Reporter(BaseReporter):
    reports_dir = _TMP_DIR
    report_builder_cls = Builder


class ReporterTest(unittest.TestCase):
    def test_xlsx_builder(self):
        with NamedTemporaryFile(suffix='.xlsx', mode='w',
                                delete=_DELETE_TMP_FILE) as tmp:
            b = Builder(tmp.name, {
                'foo': 'FOO',
                'bar': 'BAR'
            })

            b.make()

            wb = load_workbook(tmp.name)
            self.assertEqual(list(read_xlsx(wb)), [[
                ['Template', 'BAR'],
                ['FOO', None],
            ]])

    def test_reporter(self):
        reporter = Reporter()
        f, _ = reporter.get_report({'foo': 'FOO', 'bar': 'BAR'})
        wb = load_workbook(f)
        self.assertEqual(list(read_xlsx(wb)), [[
            ['Template', 'BAR'],
            ['FOO', None],
        ]])

        os.remove(f)
